@@ -1,6 +1,6 @@
 package Mojo::IRC::Server;
 use strict;
-$Mojo::IRC::Server::VERSION = "1.7.0";
+$Mojo::IRC::Server::VERSION = "1.7.1";
 use Encode;
 use Encode::Locale;
 use Carp;
@@ -15,6 +15,7 @@ use Mojo::IRC::Server::Channel;
 
 has host => "0.0.0.0";
 has port => 6667;
+has listen => undef;
 has network => "Mojo IRC NetWork";
 has ioloop => sub { Mojo::IOLoop->singleton };
 has parser => sub { Parse::IRC->new };
@@ -99,17 +100,21 @@ sub new_user{
     $user->io->on(error=>sub{
         my ($stream, $err) = @_;
         $s->emit(close_user=>$user);
+        $user->emit("close");
         $s->debug("C[" .$user->name."] 连接错误: $err");
     });
     $user->io->on(close=>sub{
         my ($stream, $err) = @_;
         $s->emit(close_user=>$user);
+        $user->emit("close");
     });
     $user->on(nick=>sub{my($user,$msg) = @_;my $nick = $msg->{params}[0];$user->set_nick($nick)});
     $user->on(user=>sub{my($user,$msg) = @_;
         if(defined $user->search_user(user=>$msg->{params}[0])){
             $user->send($user->serverident,"446",$user->nick,"该帐号已被使用");
-             return;
+            $user->io->close_gracefully();
+            $user->{_server}->remove_user($user);
+            return;
         }
         $user->user($msg->{params}[0]);
         #$user->mode($msg->{params}[1]);
@@ -163,8 +168,8 @@ sub new_user{
             my $channel_name = $msg->{params}[0];
             my $content = $msg->{params}[1];
             my $channel = $user->search_channel(name=>$channel_name);
-            if(not defined $channel){$user->send($user->serverident,"403",$channel_name,"No such channel");return}
-            $user->forward($user->ident,"PRIVMSG",$channel_name,$content);
+            if(not defined $channel){$user->send($user->serverident,"403",$user->nick,$channel_name,"No such channel");return}
+            $channel->forward($user,$user->ident,"PRIVMSG",$channel_name,$content);
             $s->info({level=>"IRC频道消息",title=>$user->nick ."|" .$channel->name.":"},$content);
         }
         else{
@@ -185,7 +190,7 @@ sub new_user{
             my $channel_name = $msg->{params}[0];
             my $channel_mode = $msg->{params}[1];
             my $channel = $user->search_channel(name=>$channel_name);
-            if(not defined $channel){$user->send($user->serverident,"403",$channel_name,"No such channel");return}
+            if(not defined $channel){$user->send($user->serverident,"403",$user->nick,$channel_name,"No such channel");return}
             if(defined $channel_mode and $channel_mode eq "b"){
                 $user->send($user->serverident,"368",$user->nick,$channel_name,"End of channel ban list");
             }   
@@ -204,7 +209,7 @@ sub new_user{
     $user->on(who=>sub{my($user,$msg) = @_;
         my $channel_name = $msg->{params}[0];
         my $channel = $user->search_channel(name=>$channel_name);
-        if(not defined $channel){$user->send($user->serverident,"403",$channel_name,"No such channel");return}
+        if(not defined $channel){$user->send($user->serverident,"403",$user->nick,$channel_name,"No such channel");return}
         $user->send($user->serverident,"352",$user->nick,$channel_name,$user->user,$user->host,$user->servername,$user->nick,"H","0 " . $user->realname);
         $user->send($user->serverident,"315",$user->nick,$channel_name,"End of WHO list");
     });
@@ -235,7 +240,7 @@ sub add_channel{
     my $is_cover = shift;
     my $channel_name = $channel->name;
     $channel_name = "#" . $channel_name if substr($channel_name,0,1) ne "#";
-    $channel_name=~s/\s+|,|&//g;
+    $channel_name=~s/\s|,|&//g;
     $channel->name($channel_name);
     my $c = $s->search_channel(name=>$channel->name);
     return $c if defined $c;
@@ -250,7 +255,7 @@ sub add_user{
     my $is_cover = shift;
     if($user->is_virtual){
         my $nick = $user->nick;
-        $nick =~s/\s+|\@|!//g;$nick = '未知昵称' if not $nick;
+        $nick =~s/\s|\@|!//g;$nick = '未知昵称' if not $nick;
         $user->nick($nick);
         while(1){
             my $u = $s->search_user(nick=>$user->nick);
@@ -275,6 +280,7 @@ sub remove_user{
     my $user = shift;
     for(my $i=0;$i<@{$s->user};$i++){
         if($user->id eq $s->user->[$i]->id){
+            $_->remove_user($s->user->[$i]->id) for $s->user->[$i]->channels;
             splice @{$s->user},$i,1;
             if($user->is_virtual){
                 $s->info("c[".$user->name."] 已被移除");
@@ -344,25 +350,33 @@ sub ident {
 }
 sub ready {
     my $s = shift;
-    $s->ioloop->server({host=>$s->host,port=>$s->port}=>sub{
-        my ($loop, $stream) = @_;
-        $stream->timeout(0);
-        my $id = join ":",(
-            $stream->handle->sockhost,
-            $stream->handle->sockport,
-            $stream->handle->peerhost,
-            $stream->handle->peerport
-        );
-        my $user = $s->new_user(
-            id      =>  $id,
-            name    =>  join(":",($stream->handle->peerhost,$stream->handle->peerport)),
-            io      =>  $stream,
-        );
-        $user->host($s->clienthost) if defined $s->clienthost;
-        
-        $s->emit(new_user=>$user);
-    });
-
+    my @listen = ();
+    if(defined $s->listen and ref $s->listen eq "ARRAY"){
+        push @listen,{host=>$_->{host} || "0.0.0.0",port=>$_->{port}||"6667"} for @{$s->listen} ;
+    }
+    else{
+        @listen = ({host=>$s->host,port=>$s->port});
+    }
+    for my $listen (@listen){
+        $s->ioloop->server({host=>$listen->{host},port=>$listen->{port}}=>sub{
+            my ($loop, $stream) = @_;
+            $stream->timeout(0);
+            my $id = join ":",(
+                $stream->handle->sockhost,
+                $stream->handle->sockport,
+                $stream->handle->peerhost,
+                $stream->handle->peerport
+            );
+            my $user = $s->new_user(
+                id      =>  $id,
+                name    =>  join(":",($stream->handle->peerhost,$stream->handle->peerport)),
+                io      =>  $stream,
+            );
+            $user->host($s->clienthost) if defined $s->clienthost;
+            $s->emit(new_user=>$user);
+        });
+    }
+    
     $s->on(new_user=>sub{
         my ($s,$user)=@_;
         $s->debug("C[".$user->name. "]已连接");
@@ -377,7 +391,6 @@ sub ready {
         my ($s,$user,$msg)=@_;
         $s->remove_user($user);
     });
-
 }
 sub run{
     my $s = shift;
